import contextlib
import os
import pathlib
import sys
import textwrap

from gather.commands import add_argument
from . import ENTRY_DATA


@ENTRY_DATA.register(add_argument("--dotfiles", required=True), name="bash-init")
def shell_init(args):  # pragma: no cover
    home = pathlib.Path(args.env["HOME"])
    dotfiles_bin = pathlib.Path(args.dotfiles) / "bin"
    local_bin = home / ".local" / "bin"
    workon_home = home / "venv"
    target_path = args.env["PATH"]
    for bin_path in [dotfiles_bin, local_bin]:
        path_name = os.fspath(bin_path)
        if path_name in os.get_exec_path(args.env):
            continue
        target_path = f"{path_name}:{target_path}"
    venvwrapper = pathlib.Path(sys.prefix) / "bin" / "virtualenvwrapper.sh"
    print(f"export WORKON_HOME={workon_home}")
    print(f"source {os.fspath(venvwrapper)}")
    args.safe_run(["starship", "init", "bash"], capture_output=False)
    print(f"export PATH={target_path}")
    print(
        textwrap.dedent(
            """\
    function set_win_title(){
        echo -ne "\033]0; $(basename "$PWD") ${VIRTUAL_ENV_PROMPT} \007";
    }
    starship_precmd_user_func="set_win_title"
    """
        )
    )


@ENTRY_DATA.register(
    add_argument("--dotfiles", required=True),
    add_argument("--no-dry-run", action="store_true", default=False),
    name="bash-install",
)
def install(args):  # pragma: no cover
    config = pathlib.Path(args.env["HOME"]) / ".config"
    dotfiles_config = pathlib.Path(args.dotfiles) / "config"
    if config.is_symlink():
        if args.no_dry_run:
            config.unlink()
    if config.exists():
        raise SystemExit("Cannot set up config directory", config)
    if args.no_dry_run:
        config.symlink_to(dotfiles_config)
    bash_profile = pathlib.Path(args.env["HOME"]) / ".bash_profile"
    python = sys.executable
    bash_init = f'eval "$({python} -m dotmoshez bash-init --dotfiles {args.dotfiles})"'
    ssh_agent = f'eval "$({python} -m dotmoshez ssh-agent-env --no-dry-run)"'
    with contextlib.ExitStack() as stack:
        if args.no_dry_run:
            fpout = stack.enter_context(bash_profile.open("a"))
        else:
            fpout = sys.stdout
        print(bash_init, file=fpout)
        print(ssh_agent, file=fpout)



import os
import pathlib

from . import ENTRY_DATA
from gather.commands import add_argument


@ENTRY_DATA.register(
    add_argument("--no-dry-run", action="store_true", default=True),
    add_argument("--container", required=True),
    add_argument("--host", required=True),
    add_argument("--user", required=True),
)
def sshirc(args):  # pragma: no cover
    execlp = getattr(args, "execlp", os.execlp)
    command = ["tmux", "at"]
    dockerize = [
        "sudo",
        "docker",
        "exec",
        "-it",
        "-u",
        args.user,
        args.container,
    ] + command
    ssh_opts = []
    for key, value in dict(ServerAliveInterval=5, ServerAliveCountMax=2).items():
        ssh_opts.extend(["-o", f"{key}={value}"])
    sshize = ["ssh", "ssh", *ssh_opts, "-t", f"{args.user}@{args.host}"] + dockerize
    execlp(*sshize)


def parse_environment(env_path):  # pragma: no cover
    try:
        content = env_path.read_text()
    except OSError:
        content = ""
    for line in content.replace(";", "\n").splitlines():
        line = line.split("#", 1)[0]
        line = line.strip()
        if line == "":
            continue
        try:
            key, value = line.split("=", 1)
        except ValueError:
            continue
        yield key, value


def is_agent_up(env, safe_run):  # pragma: no cover
    try:
        pid = env["SSH_AGENT_PID"]
    except KeyError:
        return False
    results = safe_run(["ps", "-ef", "--width", "1000"])
    lines = iter(results.stdout.splitlines())
    fields = next(lines).split()
    for a_line in lines:
        if "<defunct>" in a_line:
            continue
        values = dict(zip(fields, a_line.split()))
        if values["PID"] == pid:
            return True
    return False


def bring_up_agent(env_path, run):  # pragma: no cover
    results = run(["ssh-agent"], capture_output=True)
    agent_output = results.stdout
    agent_output = agent_output.replace("echo ", "# echo ")
    env_path.write_text(agent_output)


@ENTRY_DATA.register(
    add_argument("--no-dry-run", action="store_true", default=False),
    add_argument(
        "--env-path", default=pathlib.Path.home() / ".ssh" / "agent-environment"
    ),
    name="ssh-agent-env",
)
def ssh_agent_env(args):  # pragma: no cover
    args.env_path = pathlib.Path(args.env_path)
    env = dict(parse_environment(args.env_path))
    if not is_agent_up(env, args.safe_run):
        bring_up_agent(args.env_path, args.run)
    print(args.env_path.read_text())



import unittest
from hamcrest import assert_that, contains_string

from .. import __version__
from .. import clean_ipynb
from .. import init_py
from .. import git_extras
from .. import ssh
from .. import bash


class TestInit(unittest.TestCase):
    def test_version(self):
        assert_that(__version__, contains_string("."))

    def test_clean_ipynb(self):
        assert_that(clean_ipynb.__doc__ or "", contains_string(""))

    def test_init_py(self):
        assert_that(init_py.__doc__ or "", contains_string(""))

    def test_ssh(self):
        assert_that(ssh.__doc__ or "", contains_string(""))

    def test_bash(self):
        assert_that(bash.__doc__ or "", contains_string(""))

    def test_git_extras(self):
        assert_that(git_extras.__doc__ or "", contains_string(""))



import pathlib
import os

from . import ENTRY_DATA
from gather.commands import add_argument


def make_git(args):  # pragma: no cover
    def git(*cmdargs):
        return args.run(["git", *cmdargs], cwd=args.env["PWD"])

    return git


@ENTRY_DATA.register(
    add_argument("--no-dry-run", action="store_true", default=True),
    name="git-sync",
)
def git_sync(args):  # pragma: no cover
    git = make_git(args)
    git("add", ".")
    git("commit", "-a", "-m", "checkpoint")
    git("push")


@ENTRY_DATA.register(
    add_argument("--no-dry-run", action="store_true", default=True),
    add_argument("name"),
    name="git-create",
)
def git_create(args):  # pragma: no cover
    git = make_git(args)
    target_dir = pathlib.Path(args.env["HOME"]) / "src" / args.name
    target_dir.mkdir(parents=True)
    args.env = dict(args.env, PWD=os.fspath(target_dir))
    git("init", ".")
    git("remote", "add", "origin", f"git@github.com:moshez/{args.name}")
    git("push", "-u", "origin", "trunk")



import os
import pathlib
import shutil

from . import ENTRY_DATA
from gather.commands import add_argument


@ENTRY_DATA.register(
    add_argument("--no-dry-run", action="store_true", default=True),
    name="clean-ipynb",
)
def clean_ipynb(args):  # pragma: no cover
    for root, dirs, files in os.walk(args.env["PWD"]):
        ipynb_cp = pathlib.Path(root) / ".ipynb_checkpoints"
        if not ipynb_cp.is_dir():
            continue
        shutil.rmtree(ipynb_cp)



import pathlib
import sys
from typing import Optional
import subprocess
import logging

import attrs
import hyperlink

from . import ENTRY_DATA
from gather.commands import add_argument

LOGGER = logging.getLogger(__name__)


@attrs.frozen
class DetailsBuilder:  # pragma: no cover
    name: Optional[str] = attrs.field(default=None)
    organization: Optional[str] = attrs.field(default=None)
    description: Optional[str] = attrs.field(default=None)
    maintainer_name: Optional[str] = attrs.field(default=None)
    maintainer_email: Optional[str] = attrs.field(default=None)

    def inform(self, name, value):
        if getattr(self, name) is not None:
            return self
        return attrs.evolve(self, **{name: value})

    def data(self):
        res = attrs.asdict(self)
        not_ready = [key for key, value in res.items() if value is None]
        if len(not_ready) != 0:
            raise ValueError("not all values retrieved", not_ready)
        return res


def parse_args(db, args):  # pragma: no cover
    for field in attrs.fields(type(db)):
        db = db.inform(field.name, getattr(args, field.name))
    return db


def parse_remote(db, git_remote_output):  # pragma: no cover
    link = hyperlink.parse(git_remote_output.strip())
    organization, name = link.path[-2:]
    db = db.inform("organization", organization)
    db = db.inform("name", name.removesuffix(".git"))
    return db


def parse_user(db, git_user_cofig):  # pragma: no cover
    details = dict(line.split(None, 1) for line in git_user_cofig.splitlines())
    for key, value in details.items():
        db = db.inform("maintainer_" + key.removeprefix("user."), value)
    return db


def get_details(args):  # pragma: no cover
    db = parse_args(DetailsBuilder(), args)
    cwd = pathlib.Path(args.env["PWD"])
    try:
        called = args.safe_run(["git", "remote", "get-url", "origin"], cwd=cwd)
    except subprocess.CalledProcessError:
        has_git = False
        LOGGER.info("Git info not available")
    else:
        db = parse_remote(db, called.stdout)
        has_git = True
    called = args.safe_run(["git", "config", "--get-regexp", r"^user\."], cwd=cwd)
    db = parse_user(db, called.stdout)
    return db.data(), has_git


ARGS_TO_FIELDS = dict(
    name="project_name",
    description="short_description",
)


@ENTRY_DATA.register(
    add_argument("--no-dry-run", action="store_true", default=False),
    *[
        add_argument("--" + field.name.replace("_", "_"))
        for field in attrs.fields(DetailsBuilder)
    ],
)
def init(args):  # pragma: no cover
    data, has_git = get_details(args)
    cmdline = [
        sys.executable,
        "-m",
        "copier",
        "copy",
        "gh:moshez/python-standard.git",
        args.env["PWD"],
    ]
    cmdline += [
        f"--data={ARGS_TO_FIELDS.get(key, key)}={value}" for key, value in data.items()
    ]
    args.run(
        cmdline,
        capture_output=False,
    )
    if not has_git:
        url = f"https://github.com/{data['organization']}/{data['name']}"
        args.run(["git", "init", "."], cwd=args.env["PWD"])
        args.run(["git", "remote", "add", "origin", url], cwd=args.env["PWD"])
        args.run(["git", "commit", "--allow-empty", "-m", "Initial commit"])
        args.run(["git", "push", "--set-upstream", "origin", "trunk"])
